{***************************************************************************}
{ Property Editor                                               }
{ for Delphi
{ version 1.1
{                                                                           }
{ written by Jeong Byoung Sun                                               }
{            copyright  2004 - 2011                                        }
{            Email : hanuleyewa@paran.com                                   }
{                                                                           }
{ The source code is given as is. The author is not responsible             }
{ for any possible damage done due to the use of this code.                 }
{ The component can be freely used in any application. The complete         }
{ source code remains property of the author and may not be distributed,    }
{ published, given or sold in any form as such. No parts of the source      }
{ code can be included in any other component or application without        }
{ written authorization of the author.                                      }
{***************************************************************************}

unit DBLRegS;

interface

uses
  SysUtils, Classes, DBClient, Provider, FldLinks, DBReg,
  DBConsts, DesignIntf, DesignEditors, SqlExpr, TreeIntf,
  DsnDbCst, MidConst, DB, DBLocalS, StrEdit,
  {$IFDEF MSWINDOWS}SqlReg{$ENDIF}{$IFDEF LINUX}DBWaiter{$ENDIF};
type

 TPropertyType = (typeCommandText, typeIndexNames, typeSortFields, typeKeyNames, typeConnectionNames);

{ Property Editors }

  TUpdateSQLProperty = class(TStringListProperty)
  private
    FUpdateKind: TUpdateKind;
  protected

  public
    procedure Activate; override;
    procedure EditSQLText; virtual;
    procedure Edit; override;
    function GetValue: string; override;
    property UpdateKind: TUpdateKind read FUpdateKind write FUpdateKind;
  end;

  TModifySQLProperty = class(TUpdateSQLProperty)
  public
    procedure Activate; override;
  end;

  TInsertSQLProperty = class(TUpdateSQLProperty)
  public
    procedure Activate; override;
  end;

  TDeleteSQLProperty = class(TUpdateSQLProperty)
  public
    procedure Activate; override;
  end;

{TCommandTextEditor }
  TCommandTextEditor = class(TDBStringProperty)
  private
    FCommandList: TStrings;
    FCommandType: TSQLCommandType;
    function GetDataSet: TCustomSQLDataSet;
    function GetProvider: TDataSetProvider;
    function GetCommandText: String;
    function GetConnection(Opened: Boolean): TCustomConnection;
    procedure SetCommandText(Value: String);
  protected
    function GetStrings: TStrings; virtual;
    procedure SetStrings(const Value: TStrings); virtual;
  public
    constructor Create(const ADesigner: IDesigner; APropCount: Integer); override;
    procedure Activate; override;
    function AutoFill: Boolean; override;
    destructor Destroy; override;
    procedure Edit; override;
    procedure EditSQLText; virtual;
    procedure EditCommandText; virtual;
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValueList(List: TStrings); override;
    property CommandType: TSQLCommandType read FCommandType write FCommandType;
  end;

procedure Register;

function LoadSQLClientDatasetFromFile(const ADataset: TClientDataset): Boolean;

implementation

uses
  TypInfo, Consts, SqlEdit, DBCommon, DBLocal,
  Forms, Controls, StringsEdit, UpdateSQLS, UpdateProvider, Dialogs,
  {$IFDEF MSWINDOWS}CDSEdit{$ENDIF}
  {$IFDEF LINUX}CDSEditLin{$ENDIF};

function LoadSQLClientDatasetFromFile(const ADataset: TClientDataset): Boolean;
var
  FileName: string;
begin
  FileName := GetLoadFromFileName;
  if FileName <> '' then
    TSQLClientDataset(ADataSet).LoadFromFile(FileName);
  Result := FileName <> '';
end;

{ TSQLStringListProperty }

procedure TUpdateSQLProperty.EditSQLText;
var
  Command: string;
  UpdateSQL: TCustomdbxUpdateSQL;
  Success: Boolean;
  Q: string;
  TblName: string;
begin
  if paDialog in GetAttributes then
  begin
    UpdateSQL := GetComponent(0) as TCustomdbxUpdateSQL;
    Command := GetStrings.Text;
    try
      if Command <> '' then
        TblName := GetTableNameFromSQL(Command);
      Q := UpdateSQL.SQLQuery[FUpdateKind].GetQuoteChar;
      Success := EditSQL(Command, UpdateSQL.SQLConnection.GetTableNames,
                  UpdateSQL.SQLConnection.GetFieldNames, TblName, Q);
      if Success then
      begin
        SetStrValue(Command);
        UpdateSQL.SQL[FUpdateKind].Text:= Command;
      end;
    finally
      // ignore exception in IDE
    end;
  end;
end;

procedure TUpdateSQLProperty.Edit;
var
  UpdateSQL: TCustomdbxUpdateSQL;
begin
  UpdateSQL:= GetComponent(0) as TCustomdbxUpdateSQL;
  if (UpdateSQL.SQLConnection <> nil) then
  begin
    UpdateSQL.SQLConnection.Open;
    EditSQLText;
  end else
    inherited;
end;

function TUpdateSQLProperty.GetValue: string;
begin
  Result:= inherited GetValue;
  {if (GetComponent(0) as TdbxUpdateSQL).SQL[FUpdateKind].Count > 0 then
    FmtStr(Result, '(%s)', [UpperCase(GetPropType^.Name)])
  else
    Result:= inherited GetValue;}
end;

procedure TUpdateSQLProperty.Activate;
begin
  inherited;
end;

{ TModifySQLProperty }

procedure TModifySQLProperty.Activate;
begin
  inherited;
  UpdateKind:= ukModify;
end;
{ TInsertSQLProperty }

procedure TInsertSQLProperty.Activate;
begin
  inherited;
  UpdateKind:= ukInsert;
end;

{ TDeleteSQLProperty }

procedure TDeleteSQLProperty.Activate;
begin
  inherited;
  UpdateKind:= ukDelete;
end;

{ TCommandTextEditor }

type
  TAccessDataSet = class(TCustomClientDataSet);

constructor TCommandTextEditor.Create(const ADesigner: IDesigner;
  APropCount: Integer);
begin
  inherited;
  FCommandList:= TStringList.Create;
end;

destructor TCommandTextEditor.Destroy;
begin
  FCommandList.Free;
  inherited;
end;

function TCommandTextEditor.GetAttributes: TPropertyAttributes;
begin
  if (CommandType in [ctTable, ctStoredProc]) then
    Result := [paValueList, paSortList, paMultiSelect] else {Drop down list for name list}
    Result := [paMultiSelect, paRevertable, paDialog]; {SQL or File}
{$IFDEF LINUX}
  Result := Result + [paVCL]; 
{$ENDIF}
end;

procedure TCommandTextEditor.Activate;
begin
  if GetComponent(0) is TSQLClientDataSet then
    CommandType := TSQLCommandType(TSQLClientDataSet(GetComponent(0)).CommandType)
  else
    CommandType := TSQLCommandType(ctQuery);
end;

procedure TCommandTextEditor.EditSQLText;
var
  Command: string;
  Connection: TCustomConnection;
  Success: Boolean;
  Q: string;
  TblName: string;
begin
  if paDialog in GetAttributes then
  begin
    Command := GetStrValue;
    Connection := GetConnection(True);
    if Connection = nil then DatabaseError(sConnectionMissing);
    try
      if GetCommandText <> '' then
        TblName := GetTableNameFromSQL(GetCommandText);
      Q := GetDataSet.GetQuoteChar;
      Success := EditSQL(Command, TSQLConnection(Connection).GetTableNames, 
                  TSQLConnection(Connection).GetFieldNames, TblName, Q);
      if Success then
      begin
        SetStrValue(Command);
        SetCommandText(Command);
      end;
    finally
      // ignore exception in IDE
    end;
  end;
end;

procedure TCommandTextEditor.GetValueList(List: TStrings);
var
  Connection: TSQLConnection;
begin
  Connection := TSQLConnection(GetConnection(True));
  if not Assigned(Connection) then DatabaseError(sConnectionMissing);
  try
    case CommandType of
      ctTable:
        Connection.GetTableNames(List);
      ctStoredProc:
        Connection.GetProcedureNames('',List);
    end;
  finally;
  end;
end;

procedure TCommandTextEditor.Edit;
begin
  if (CommandType = ctQuery) then
  begin
    if Assigned(GetConnection(False)) then
      EditSQLText
    else
      EditCommandText;
  end else
    inherited;
end;

function TCommandTextEditor.GetConnection(Opened: Boolean): TCustomConnection;
var
  DataSet: TCustomSQLDataSet;
begin
  Result := nil;
  DataSet := GetDataSet;
  if DataSet = nil then
    Exit;
  Result := DataSet.SQLConnection;
  if not Opened then
    Exit;
  if Result <> nil then
    Result.Open;
end;

type
  TAccessSQLClientDataSet = class(TSQLClientDataSet);
  
function TCommandTextEditor.GetProvider: TDataSetProvider;
var
  Component, Provider: TComponent;
begin
  Result:= nil;
  Component:= GetComponent(0) as TComponent;
  if Component is TSQLClientDataSet then
    Result:= TAccessSQLClientDataSet(Component).Provider
  else
    with Component do
      if Assigned(Owner) then
      begin
        Provider:= Owner.FindComponent(TAccessDataSet(Component).ProviderName);
        if Assigned(Provider) and (Provider is TDataSetProvider) then
          Result:= Provider as TDataSetProvider;
      end;
end;

function TCommandTextEditor.GetDataSet: TCustomSQLDataSet;
var
  Provider: TDataSetProvider;
begin
  Result:= nil;
  Provider:= GetProvider;
  if (Provider <> nil) then
    with Provider do
      if Assigned(DataSet) and (DataSet is TCustomSQLDataSet) then
        Result:= TCustomSQLDataSet(DataSet);
end;

function TCommandTextEditor.AutoFill: Boolean;
var
  Connection: TCustomConnection;
begin
  Connection := GetConnection(False);
  if Assigned(Connection) then
  begin
    if Connection.Connected then
      Result := True
    else
    begin
      Connection.Connected := True;
      Result := True
    end;
  end else
    Result := False;
end;

function TCommandTextEditor.GetCommandText: String;
begin
  Result:= GetStrProp(GetComponent(0), 'CommandText');
end;

procedure TCommandTextEditor.SetCommandText(Value: String);
begin
  if Value <> GetCommandText then
    SetStrProp(GetComponent(0), 'CommandText', Value);
end;

function TCommandTextEditor.GetStrings: TStrings;
begin
  FCommandList.Text:= GetCommandText;
  Result := FCommandList;
end;

procedure TCommandTextEditor.SetStrings(const Value: TStrings);
begin
  FCommandList.Assign(Value);
  SetCommandText(FCommandList.Text);
end;

procedure TCommandTextEditor.EditCommandText;
begin
  with TStringsEditDlg.Create(Application) do
  begin
    try
      CodeWndBtn.Visible:= False; //ڵâ ?
      Lines.Text:= GetCommandText;
      if ShowModal = mrOk then
        SetCommandText(Lines.Text);
    finally
      Free;
    end;
  end;
end;

procedure Register;
begin
  RegisterComponents(srSQL, [TdbxUpdateSQL]);
  RegisterComponents(srSQL, [TUpdateProvider]);
  RegisterComponents('Data Access', [TSharedClientDataSet]);
  RegisterComponents('Data Access', [TRemoteClientDataSet]);
  RegisterComponents('Data Access', [TRemoteProvider]);

  RegisterPropertyEditor(TypeInfo(TStrings), TCustomdbxUpdateSQL, 'InsertSQL', TInsertSQLProperty);
  RegisterPropertyEditor(TypeInfo(TStrings), TCustomdbxUpdateSQL, 'ModifySQL', TModifySQLProperty);
  RegisterPropertyEditor(TypeInfo(TStrings), TCustomdbxUpdateSQL, 'DeleteSQL', TDeleteSQLProperty);

  RegisterComponents(srSQL, [TSQLClientDataSet]);
  RegisterPropertyEditor(TypeInfo(String), TCustomClientDataSet, 'CommandText', TCommandTextEditor);
end;

end.
