{***************************************************************************}
{ TSQLClientDataSet Component                                               }
{ for Delphi
{ version 1.1
{                                                                           }
{ written by Jeong Byoung Sun                                               }
{            copyright  2004 - 2011                                        }
{            Email : hanuleyewa@paran.com                                   }
{                                                                           }
{ The source code is given as is. The author is not responsible             }
{ for any possible damage done due to the use of this code.                 }
{ The component can be freely used in any application. The complete         }
{ source code remains property of the author and may not be distributed,    }
{ published, given or sold in any form as such. No parts of the source      }
{ code can be included in any other component or application without        }
{ written authorization of the author.                                      }
{***************************************************************************}


unit DBLocalS;

{$R-,T-,H+,X+}

interface

{$IFDEF MSWINDOWS}
uses Windows, SysUtils, Variants, Classes, Db, DBCommon, Midas,
     SqlTimSt, DBClient, DBLocal, Provider, SqlExpr, UpdateSQLS;
{$ENDIF}
{$IFDEF LINUX}
uses Libc, SysUtils, Variants, Classes, DB, DBCommon, Midas, SqlTimSt, 
     DBClient, DBLocal, Provider, SqlExpr;
{$ENDIF}

type
  TApplyUpdateMode = (amExternal, amInternal);

  TSQLClientDataSet = class;

  TCDSUpdateSQL = class(TCustomdbxUpdateSQL)
  private
    FSQLClientDataSet: TSQLClientDataSet;
  protected
    function GetConnection: TSQLConnection; override;
  published
    property ModifySQL;
    property InsertSQL;
    property DeleteSQL;
    property BeforeGetParam;
    property BeforeUpdate;
    property TransDesc;
    property SourceDataSet: TSQLClientDataSet read FSQLClientDataSet;
  end;

  { TInternalSQLDataset }

  TInternalSQLDataset = class(TCustomSQLDataSet)
  private
    FKeyFields: string;
  protected
    function PSGetDefaultOrder: TIndexDef; override;
    procedure CloseCursor; override;
  public
    constructor Create(AOwner: TComponent); override;
    function ExecSQL(ExecDirect: Boolean = False): Integer; override;
  end;

  TInternalDataSetProvider = class(TDataSetProvider)
  private
    procedure SetCommandText(const CommandText: string);
  protected
    { Event overrides }
    procedure DoBeforeGetRecords(Count: Integer; Options: Integer;
      const CommandText: WideString; var Params, OwnerData: OleVariant); override;
    procedure DoBeforeExecute(const CommandText: WideString; var Params,
      OwnerData: OleVariant); override;
  end;

{ TSQLClientDataSet }

  TSQLClientDataSet = class(TCustomClientDataSet)
  private
    {TCustomCachedDataSet}
    FProvider: TInternalDataSetProvider;
    FSqlDBType: TSqlDBType;
    {TCustomCachedDataSet}

    FApplyUpdateMode: TApplyUpdateMode;
    FCommandText: String;
    FCommitMode: TCommitMode;
    FUpdateSQL: TCDSUpdateSQL;

    FDataSet: TInternalSQLDataset;
    FSQLConnection: TSQLConnection;
    FTranConnection: TSQLConnection;
{$IFDEF MSWINDOWS}
    FStreamedActive: Boolean;
{$ENDIF}

    {*************************************************************************}
    procedure InternalBeforeUpdateRecord(Sender: TObject; SourceDS: TDataSet;
        DeltaDS: TCustomClientDataSet; UpdateKind: TUpdateKind; var Applied: Boolean);
    procedure SetCommitMode(ACommitMode: TCommitMode);
    function GetMaxBlobSize: Integer;
    procedure SetMaxBlobSize(Value: Integer);
    {*************************************************************************}
    {TCustomCachedDataSet}
    function GetBeforeUpdateRecord : TBeforeUpdateRecordEvent;
    procedure SetBeforeUpdateRecord( Value: TBeforeUpdateRecordEvent);
    function GetAfterUpdateRecord: TAfterUpdateRecordEvent;
    procedure SetAfterUpdateRecord(Value: TAfterUpdateRecordEvent);
    function GetOnGetTableName: TGetTableNameEvent;
    procedure SetOnGetTableName(Value: TGetTableNameEvent);
    function GetOnUpdateData: TProviderDataEvent;
    procedure SetOnUpdateData(Value: TProviderDataEvent);
    function GetOnUpdateError: TResolverErrorEvent;
    procedure SetOnUpdateError(Value: TResolverErrorEvent);
    {TCustomCachedDataSet}

    function GetCommandType: TSQLCommandType;
    function GetConnection: TSQLConnection;
    function GetTranConnection: TSQLConnection;
    procedure SetCommandType(const Value: TSQLCommandType);
    procedure SetConnection(const Value: TSQLConnection);
    procedure SetTranConnection(const Value: TSQLConnection);
  protected
    {TCustomCachedDataSet}
    procedure CloseCursor; override;
    function GetCommandText: string; virtual;
    function GetOptions: TProviderOptions;
    function GetUpdateMode: TUpdateMode;
    procedure SetActive(Value: Boolean); override;
    procedure SetAggregates(Value: TAggregates); override;
    procedure SetCommandText(Value: string); override;
    procedure SetOptions(Value: TProviderOptions);
    procedure SetUpdateMode(Value: TUpdateMode);
    property Provider: TInternalDataSetProvider read FProvider write FProvider;
    property SqlDBType: TSqlDBType read FSqlDBType write FSqlDBType;
    {TCustomCachedDataSet}
    {*************************************************************************}
    procedure DoAfterApplyUpdates(var OwnerData: OleVariant); override;
    procedure DoBeforeApplyUpdates(var OwnerData: OleVariant); override;
    {*************************************************************************}
    procedure Loaded; override;

    property InternalDataSet: TInternalSQLDataset read FDataSet;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    {TCustomCachedDataSet}
    procedure LoadFromFile(const AFileName: string = '');
    {TCustomCachedDataSet}
    procedure CloneCursor(Source: TCustomClientDataSet; Reset: Boolean;
      KeepSettings: Boolean = False); override;
    function GetQuoteChar: string;

    function ParamByName(const Value: string): TParam; virtual;
    function ApplyUpdates(MaxErrors: Integer): Integer; override;
  published
    {TCustomCachedDataSet}
    { overridden by TCustomCachedDataSet descendents }
    property Active;
    { public in TClientDataSet }
    property Aggregates;
    property AggregatesActive;
    property AutoCalcFields;
    property Constraints stored ConstraintsStored;
    property DisableStringTrim;
    property FetchOnDemand;
    property FieldDefs;
    property FileName;
    property Filter;
    property Filtered;
    property FilterOptions;
    property IndexDefs stored True;
    property IndexFieldNames;
    property IndexName;
    property MasterFields;
    property MasterSource;
    { from TDataSetProvider }
    property Options: TProviderOptions read GetOptions write SetOptions default [];
    { more public in TClientDataSet }
    property ObjectView;
    property PacketRecords;
    property Params;
    property ReadOnly;
    { from TDataSetProvider again}
    property UpdateMode: TUpdateMode read GetUpdateMode write SetUpdateMode default upWhereAll;
    { TClientDataSet events }
    property BeforeOpen;
    property AfterOpen;
    property BeforeClose;
    property AfterClose;
    property BeforeInsert;
    property AfterInsert;
    property BeforeEdit;
    property AfterEdit;
    property BeforePost;
    property AfterPost;
    property BeforeCancel;
    property AfterCancel;
    property BeforeDelete;
    property AfterDelete;
    property BeforeScroll;
    property AfterScroll;
    property BeforeRefresh;
    property AfterRefresh;
    property OnCalcFields;
    property OnDeleteError;
    property OnEditError;
    property OnFilterRecord;
    property OnNewRecord;
    property OnPostError;
    property OnReconcileError;
    property BeforeApplyUpdates;
    property AfterApplyUpdates;
    property BeforeGetRecords;
    property AfterGetRecords;
    property BeforeRowRequest;
    property AfterRowRequest;
    property BeforeExecute;
    property AfterExecute;
    property BeforeGetParams;
    property AfterGetParams;
    { TDataSetProvider events }
    property BeforeUpdateRecord: TBeforeUpdateRecordEvent read GetBeforeUpdateRecord
      write SetBeforeUpdateRecord;
    property AfterUpdateRecord: TAfterUpdateRecordEvent read GetAfterUpdateRecord
      write SetAfterUpdateRecord;
    property OnGetTableName: TGetTableNameEvent read GetOnGetTableName write SetOnGetTableName;
    property OnUpdateData: TProviderDataEvent read GetOnUpdateData write SetOnUpdateData;
    property OnUpdateError: TResolverErrorEvent read GetOnUpdateError write SetOnUpdateError;
    {TCustomCachedDataSet}
    property ApplyMode: TApplyUpdateMode read FApplyUpdateMode write FApplyUpdateMode default amExternal;
    property CommitMode: TCommitMode read FCommitMode write SetCommitMode Default cmNone;
    property UpdateSQL: TCDSUpdateSQL read FUpdateSQL;
    property MaxBlobSize: Integer read GetMaxBlobSize write SetMaxBlobSize;

    property CommandText: string read GetCommandText write SetCommandText;
    property CommandType: TSQLCommandType read GetCommandType write SetCommandType default ctQuery;
    property DBConnection: TSQLConnection read GetConnection write SetConnection;
    property TranConnection: TSQLConnection read GetTranConnection write SetTranConnection;
  end;

  TAccessSQLDataSet = class(TCustomSQLDataSet);

  {TCustomSharedClientDataSet}
  TCustomSharedClientDataSet = class(TCustomClientDataSet)
  private
    FApplyUpdateMode: TApplyUpdateMode;
  protected
    procedure CheckRefresh; virtual;
    procedure InternalRefresh; override;
    //function DoRowRequest(Row: OleVariant; RequestType: Integer): OleVariant; override;
    procedure DoBeforeApplyUpdates(var OwnerData: OleVariant); override;
  public
    function ApplyUpdates(MaxErrors: Integer): Integer; override;
    function ParamByName(const Value: string): TParam; virtual;
  published
    property ApplyMode: TApplyUpdateMode read FApplyUpdateMode write FApplyUpdateMode default amExternal;

    property Active;
    property Aggregates;
    property AggregatesActive;
    property AutoCalcFields;
    property CommandText;
    property ConnectionBroker;
    property Constraints;
    property DataSetField;
    property DisableStringTrim;
    property FileName;
    property Filter;
    property Filtered;
    property FilterOptions;
    property FieldDefs;
    property IndexDefs;
    property IndexFieldNames;
    property IndexName;
    property FetchOnDemand;
    property MasterFields;
    property MasterSource;
    property ObjectView;
    property PacketRecords;
    property Params;
    property ProviderName;
    property ReadOnly;
    property RemoteServer;
    property StoreDefs;
    property BeforeOpen;
    property AfterOpen;
    property BeforeClose;
    property AfterClose;
    property BeforeInsert;
    property AfterInsert;
    property BeforeEdit;
    property AfterEdit;
    property BeforePost;
    property AfterPost;
    property BeforeCancel;
    property AfterCancel;
    property BeforeDelete;
    property AfterDelete;
    property BeforeScroll;
    property AfterScroll;
    property BeforeRefresh;
    property AfterRefresh;
    property OnCalcFields;
    property OnDeleteError;
    property OnEditError;
    property OnFilterRecord;
    property OnNewRecord;
    property OnPostError;
    property OnReconcileError;
    property BeforeApplyUpdates;
    property AfterApplyUpdates;
    property BeforeGetRecords;
    property AfterGetRecords;
    property BeforeRowRequest;
    property AfterRowRequest;
    property BeforeExecute;
    property AfterExecute;
    property BeforeGetParams;
    property AfterGetParams;
  end;

{TSharedClientDataSet}
  TSharedClientDataSet = class(TCustomSharedClientDataSet);

{TRemoteClientDataSet}
  TRemoteClientDataSet = class(TCustomSharedClientDataSet)
  private
    FCommitMode: TCommitMode;
    FUpdateSQL: TSimpleUpdateSQL;
    FUserParams: TParams;
  protected
    procedure DoBeforeApplyUpdates(var OwnerData: OleVariant); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property CommitMode: TCommitMode read FCommitMode write FCommitMode default cmPage;
    property UpdateSQL: TSimpleUpdateSQL read FUpdateSQL;
    property UserParams: TParams read FUserParams write FUserParams;
  end;

implementation

uses MidConst, SqlConst, ActiveX, DSIntf, DBConsts;

const
  ProvDataSetName = 'DataSet1';

type
  TDataPacket = PSafeArray;
  pDSAttr = ^DSAttr;
  DSAttr = type Byte;

  {TSQLCDSParams}

  TSQLCDSParams = class(TParams)
  private
    FFieldName: TStrings;
  protected
    procedure ParseSelect(SQL: string);
  public
    constructor Create(Owner: TPersistent);
    Destructor Destroy; override;
  end;

  {TSQLCDSParams}
  
constructor TSQLCDSParams.Create(Owner: TPersistent);
begin
  inherited;
  FFieldName := TStringList.Create;
end;

destructor TSQLCDSParams.Destroy;
begin
  FreeAndNil(FFieldName);
  inherited;
end;

procedure TSQLCDSParams.ParseSelect(SQL: string);
const
  SSelect = 'select';
var
  FWhereFound: Boolean;
  Start: PChar;
  FName, Value: string;
  SQLToken, CurSection, LastToken: TSQLToken;
  Params: Integer;
begin
  if Pos(' ' + SSelect + ' ', LowerCase(string(PChar(SQL)+8))) > 1 then Exit;  // can't parse sub queries
  Start := PChar(ParseSQL(PChar(SQL), True));
  CurSection := stUnknown;
  LastToken := stUnknown;
  FWhereFound := False;
  Params := 0;
  repeat
    repeat
      SQLToken := NextSQLToken(Start, FName, CurSection);
      if SQLToken in [stWhere] then
      begin
        FWhereFound := True;
        LastToken := stWhere;
      end else if SQLToken in [stTableName] then
      begin
        { Check for owner qualified table name }
        if Start^ = '.' then
          NextSQLToken(Start, FName, CurSection);
      end else
      if (SQLToken = stValue) and (LastToken = stWhere) then
        SQLToken := stFieldName;
      if SQLToken in SQLSections then CurSection := SQLToken;
    until SQLToken in [stFieldName, stEnd];
    if FWhereFound and (SQLToken in [stFieldName]) then
      repeat
        SQLToken := NextSQLToken(Start, Value, CurSection);
          if SQLToken in SQLSections then CurSection := SQLToken;
      until SQLToken in [stEnd,stValue,stIsNull,stIsNotNull,stFieldName];
    if Value='?' then
    begin
      FFieldName.Add(FName);
      Inc(Params);
    end;
  until (Params = Count) or (SQLToken in [stEnd]);
end;

{ TInternalSQLDataset }

constructor TInternalSQLDataset.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  CommandType := ctQuery;
  GetMetadata := True;
end;

function TInternalSQLDataset.ExecSQL(ExecDirect: Boolean): Integer;
begin
  Result := inherited ExecSQL(ExecDirect);
end;

procedure TInternalSQLDataset.CloseCursor;
begin
  inherited;
  if Assigned(SQLConnection) then
    SQLConnection.CloseDataSets;
end;

function TInternalSQLDataset.PSGetDefaultOrder: TIndexDef;
begin
  if FKeyFields = '' then
    Result := inherited PSGetDefaultOrder
  else
  begin  // detail table default order
    Result := TIndexDef.Create(nil);
    Result.Options := [ixUnique];      // keyfield is unique
    Result.Name := StringReplace(FKeyFields, ';', '_', [rfReplaceAll]);
    Result.Fields := FKeyFields;
  end;
end;

{TCustomSharedClientDataSet}

procedure TCustomSharedClientDataSet.CheckRefresh;
var
  RecsOut: Integer;
  Options: TGetRecordOptions;
  DataPacket: TDataPacket;
begin
  Options := [grMetaData];
  DataPacket := VarToDataPacket(DoGetRecords(PacketRecords, RecsOut,
    Byte(Options), CommandText, PackageParams(Params)));
  ProviderEOF := RecsOut <> PacketRecords;
  if not Assigned(DataPacket) then DatabaseError('Missing data provider or data packet', Self);
end;

procedure TCustomSharedClientDataSet.InternalRefresh;
begin
  CheckRefresh;
  inherited;
end;

{function TCustomSharedClientDataSet.DoRowRequest(Row: OleVariant; RequestType: Integer): OleVariant;
begin
//  CheckRefresh;
  Result:= inherited DoRowRequest(Row, RequestType);
end; }

function TCustomSharedClientDataSet.ParamByName(const Value: string): TParam;
begin
  Result := Params.ParamByName(Value);
end;

procedure TCustomSharedClientDataSet.DoBeforeApplyUpdates(
  var OwnerData: OleVariant);
begin
  inherited;
end;

function TCustomSharedClientDataSet.ApplyUpdates(
  MaxErrors: Integer): Integer;
begin
  if FApplyUpdateMode = amInternal then
  begin
    Result:= 0;
    CheckBrowseMode;
    DSBase.AcceptChanges;
    UpdateCursorPos;
    Resync([]);
  end else
    Result:= Inherited ApplyUpdates(MaxErrors);
end;

{ TSQLClientDataSet }

constructor TSQLClientDataSet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FProvider := TInternalDataSetProvider.Create(nil);
  FProvider.Name := Self.Name + 'Provider1';
  FProvider.Options := [];
  FProvider.UpdateMode := upWhereAll;
  SetProvider(FProvider);
  FDataSet := TInternalSQLDataSet.Create(Self);
  FDataSet.Name := Self.Name + ProvDataSetName;
  Provider.DataSet := FDataSet;
  FDataSet.SetSubComponent(True);
  FCommitMode:= cmNone;
  SqlDBType := typeDBX;
  FUpdateSQL:= TCDSUpdateSQL.Create(Self);
  FUpdateSQL.Name := Self.Name + 'InternalUpdateSQL1';			{ Do not localize }
  FUpdateSQL.FSQLClientDataSet:= Self;
  FUpdateSQL.SetSubComponent(True);
end;

destructor TSQLClientDataSet.Destroy;
begin
  if Assigned(FDataSet) then
    FreeAndNil(FDataSet);
  if Assigned(FUpdateSQL) then
     FreeAndNil(FUpdateSQL);
  if Assigned(FProvider) then
     FreeAndNil(FProvider);
  inherited Destroy;
end;

function TSQLClientDataSet.GetCommandType: TSQLCommandType;
begin
  Result := FDataSet.CommandType;
end;

procedure TSQLClientDataSet.SetCommandType(const Value: TSQLCommandType);
begin
  if Value <> CommandType then
  begin
    FDataSet.CommandType := Value;
    if not (csLoading in ComponentState) then
      CommandText := '';
  end;
end;

function TSQLClientDataSet.GetCommandText: string;
begin
  Result := FCommandText;
end;

procedure TSQLClientDataSet.SetCommandText(Value: string);
begin
  if Value <> FCommandText then
  begin
    FCommandText:= Value;
    inherited SetCommandText(FCommandText);
    if not (poAllowCommandText in Provider.Options) and (FCommandText <> FDataSet.CommandText) then
      FDataSet.CommandText:= FCommandText;
  end;
end;

function TSQLClientDataSet.GetConnection: TSQLConnection;
begin
  if Assigned(FSQLConnection) then Result := FSQLConnection
  else Result := FDataSet.SQLConnection as TSQLConnection;
end;

procedure TSQLClientDataSet.SetConnection(const Value: TSQLConnection);
begin
  if FDataSet.SQLConnection <> Value then
  begin
    CheckInactive;
    FDataSet.SQLConnection := Value;
  end;
end;

function TSQLClientDataSet.GetQuoteChar: string;
begin
  if Assigned(FDataSet) then
    Result := FDataSet.GetQuoteChar;
end;

procedure TSQLClientDataSet.SetActive(Value: Boolean);

  procedure CheckMasterSourceActive(const MasterSource: TDataSource);
  begin
    if Assigned(MasterSource) and Assigned(MasterSource.DataSet) then
      if not MasterSource.DataSet.Active then
        DatabaseError(SMasterNotOpen);
  end;

  procedure SetDetailsActive(const Value: Boolean);
  var
    DetailList: TList;
    I: Integer;
  begin
    DetailList := TList.Create;
    try
      GetDetailDataSets(DetailList);
      for I := 0 to DetailList.Count -1 do
      if TDataSet(DetailList[I]) is TSQLClientDataSet then
        TSQLClientDataSet(TDataSet(DetailList[I])).Active := Value;
    finally
      DetailList.Free;
    end;
  end;

begin
  if Value then
  begin
    if csLoading in ComponentState then
    begin
  {$IFDEF MSWINDOWS}
      FStreamedActive := True;
  {$ENDIF}
      exit;
    end;
  end;
  if Value and (FDataSet.ObjectView <> ObjectView) then
    FDataSet.ObjectView := ObjectView;

  if Value then
  begin
    if (FileName <> '') and (CommandText = '') then
      SetProvider(Nil)
    else if Assigned(FProvider) then
      SetProvider(FProvider);
  end;
  inherited SetActive(Value);
  if (not Value) and Assigned(FProvider) then
    SetProvider(FProvider);

  SetDetailsActive(Value);
end;

procedure TSQLClientDataSet.CloneCursor(Source: TCustomClientDataSet; Reset: Boolean;
   KeepSettings: Boolean = False); 
begin
  if not (Source is TSQLClientDataSet) then
    DatabaseError(SInvalidClone);
  Provider.DataSet := TSQLClientDataSet(Source).Provider.DataSet;
  Provider.Options:= TSQLClientDataSet(Source).Provider.Options;
  DBConnection := TSQLClientDataSet(Source).DBConnection;
  CommandText := TSQLClientDataSet(Source).CommandText;
  inherited CloneCursor(Source, Reset, KeepSettings);
end;

function TSQLClientDataSet.ApplyUpdates(MaxErrors: Integer): Integer;
begin
  if FApplyUpdateMode = amInternal then
  begin
    Result:= 0;
    CheckBrowseMode;
    DSBase.AcceptChanges;
    UpdateCursorPos;
    Resync([]);
  end else
    Result:= Inherited ApplyUpdates(MaxErrors);
end;

procedure TSQLClientDataSet.DoAfterApplyUpdates(var OwnerData: OleVariant);
begin
  inherited;
  if (FCommitMode = cmPage) then
    FUpdateSQL.Commit;
end;

procedure TSQLClientDataSet.DoBeforeApplyUpdates(
  var OwnerData: OleVariant);
begin
  if Not Assigned(BeforeUpdateRecord) then
    BeforeUpdateRecord:= InternalBeforeUpdateRecord;
  if (FCommitMode = cmPage) then
    FUpdateSQL.StartTransaction;
  inherited;
end;

procedure TSQLClientDataSet.InternalBeforeUpdateRecord(Sender: TObject;
  SourceDS: TDataSet; DeltaDS: TCustomClientDataSet;
  UpdateKind: TUpdateKind; var Applied: Boolean);
begin
  if DeltaDS.UpdateStatus = usUnmodified then
    Exit;
  if UpdateSQL.SQL[UpdateKind].Text = '' then
    Exit;
  Applied:= False;
  FUpdateSQL.DataSet:= DeltaDS;

  if FCommitMode = cmRecord then
    FUpdateSQL.StartTransaction;
  try
    FUpdateSQL.Apply(UpdateKind);
    if FCommitMode = cmRecord then
      FUpdateSQL.Commit;
    Applied:= True;
  except
    on E: Exception do
    begin
      if FCommitMode = cmRecord then
        FUpdateSQL.Rollback;
      Raise ExceptionClass(E.ClassType).Create(E.Message);
    end;
  end;
end;

procedure TSQLClientDataSet.SetCommitMode(ACommitMode: TCommitMode);
begin
  if FCommitMode <> ACommitMode then
  begin
    FCommitMode:= ACommitMode;
    if FCommitMode = cmNone then
      Exit;
    if csLoading in ComponentState then
      Exit;
    if Not (Assigned(FDataSet) and Assigned(FUpdateSQL.SQLConnection) and Assigned(FDataSet.SQLConnection))then
    begin
      FCommitMode:= cmNone;
      Exit;
    end;
  end;
end;

function TSQLClientDataSet.ParamByName(const Value: string): TParam;
begin
  Result := Params.ParamByName(Value);
end;

function TSQLClientDataSet.GetMaxBlobSize: Integer;
begin
  Result:= FDataSet.MaxBlobSize;
end;

procedure TSQLClientDataSet.SetMaxBlobSize(Value: Integer);
begin
  FDataSet.MaxBlobSize:= Value;
end;

procedure TSQLClientDataSet.CloseCursor;
begin
  inherited;
  SetProvider(FProvider);
end;

function TSQLClientDataSet.GetAfterUpdateRecord: TAfterUpdateRecordEvent;
begin
  Result := FProvider.AfterUpdateRecord;
end;

function TSQLClientDataSet.GetBeforeUpdateRecord: TBeforeUpdateRecordEvent;
begin
  Result := FProvider.BeforeUpdateRecord;
end;

function TSQLClientDataSet.GetOnGetTableName: TGetTableNameEvent;
begin
  Result := FProvider.OnGetTableName;
end;

function TSQLClientDataSet.GetOnUpdateData: TProviderDataEvent;
begin
  Result := FProvider.OnUpdateData;
end;

function TSQLClientDataSet.GetOnUpdateError: TResolverErrorEvent;
begin
  Result := FProvider.OnUpdateError;
end;

function TSQLClientDataSet.GetOptions: TProviderOptions;
begin
  Result := FProvider.Options;
end;

function TSQLClientDataSet.GetUpdateMode: TUpdateMode;
begin
  Result := FProvider.UpdateMode;
end;

procedure TSQLClientDataSet.Loaded;
begin
  inherited Loaded;
{$IFDEF MSWINDOWS}
  if FStreamedActive then
  begin
    try 
      FStreamedActive := False;
      SetActive(True);
    except
      if csDesigning in ComponentState then
        InternalHandleException else
        raise;
    end;
  end;  
{$ENDIF}
end;

procedure TSQLClientDataSet.LoadFromFile(const AFileName: string);
var
  SaveFile: string;
begin
  SaveFile := AFileName;
  inherited FileName := AFileName;
  inherited LoadFromFile(AFileName);
  inherited FileName := SaveFile;
end;

procedure TSQLClientDataSet.SetAfterUpdateRecord(
  Value: TAfterUpdateRecordEvent);
begin
  FProvider.AfterUpdateRecord := Value;
end;

procedure TSQLClientDataSet.SetAggregates(Value: TAggregates);
begin
  if Active then DatabaseError(SDataSetOpen);
  inherited SetAggregates(Value);
end;

procedure TSQLClientDataSet.SetBeforeUpdateRecord(
  Value: TBeforeUpdateRecordEvent);
begin
  FProvider.BeforeUpdateRecord := Value;
end;

procedure TSQLClientDataSet.SetOnGetTableName(Value: TGetTableNameEvent);
begin
  FProvider.OnGetTableName := Value;
end;

procedure TSQLClientDataSet.SetOnUpdateData(Value: TProviderDataEvent);
begin
  FProvider.OnUpdateData := Value;
end;

procedure TSQLClientDataSet.SetOnUpdateError(Value: TResolverErrorEvent);
begin
  FProvider.OnUpdateError := Value;
end;

procedure TSQLClientDataSet.SetOptions(Value: TProviderOptions);
begin
  FProvider.Options:= Value;
end;

procedure TSQLClientDataSet.SetUpdateMode(Value: TUpdateMode);
begin
  FProvider.UpdateMode := Value;
end;

function TSQLClientDataSet.GetTranConnection: TSQLConnection;
begin
  if Assigned(FTranConnection) then Result := FTranConnection
  else Result := DBConnection;
end;

procedure TSQLClientDataSet.SetTranConnection(const Value: TSQLConnection);
begin
  if FTranConnection <> Value then
  begin
    FTranConnection:= Value;

  end;
end;

{ TCDSUpdateSQL }

function TCDSUpdateSQL.GetConnection: TSQLConnection;
begin
  Result:= FSQLClientDataSet.TranConnection;
end;

{ TRemoteClientDataSet }

constructor TRemoteClientDataSet.Create(AOwner: TComponent);
begin
  inherited;
  FCommitMode:= cmPage;
  FUpdateSQL:= TSimpleUpdateSQL.Create(Self);
  FUpdateSQL.Name:= 'InternalUpdateSQL1';
  FUpdateSQL.SetSubComponent(True);
  FUserParams:= TParams.Create;
end;

destructor TRemoteClientDataSet.Destroy;
begin
  FUpdateSQL.Free;
  FUserParams.Free;
  inherited;
end;

procedure TRemoteClientDataSet.DoBeforeApplyUpdates(
  var OwnerData: OleVariant);
var
  i: Integer;
  uk: TUpdateKind;
begin
  OwnerData:= VarArrayCreate([0, 3 +FUserParams.Count], VarVariant);
  for uk:= Low(TUpdateKind) to High(TUpdateKind) do
    OwnerData[Ord(uk)]:= FUpdateSQL.SQL[uk].Text;
  OwnerData[3]:= Ord(FCommitMode);
  if FUserParams.Count > 0 then
  begin
    for i:= 0 to FUserParams.Count -1 do
      with FUserParams[i] do
        OwnerData[4 +i]:= VarArrayOf([Name, DataType, ParamType, Value]);
  end;
  inherited;
end;

{ TInternalDataSetProvider }

procedure TInternalDataSetProvider.SetCommandText(const CommandText: string);
begin
  if CommandText = '' then Exit;
  if not (poAllowCommandText in Options) then
    DatabaseError(SCannotChangeCommandText);
  CheckDataSet;
  IProviderSupport(DataSet).PSSetCommandText(CommandText);
end;

procedure TInternalDataSetProvider.DoBeforeExecute(
  const CommandText: WideString; var Params, OwnerData: OleVariant);
var
  FCommand: String;
begin
  if not (poAllowCommandText in Options) then
    FCommand:= ''
  else
    FCommand:= CommandText;
  SetCommandText(FCommand);
  SetParams(Params);
  inherited DoBeforeExecute(FCommand, Params, OwnerData);
end;

procedure TInternalDataSetProvider.DoBeforeGetRecords(Count,
  Options: Integer; const CommandText: WideString; var Params,
  OwnerData: OleVariant);
var
  FCommand: String;
begin
  if not (poAllowCommandText in Self.Options) then
    FCommand:= ''
  else
    FCommand:= CommandText;
  SetCommandText(FCommand);
  SetParams(Params);
  inherited DoBeforeGetRecords(Count, Options, FCommand, Params, OwnerData);
end;

end.
