{***************************************************************************}
{ TCustomdbxUpdateSQL Component                                             }
{ for Delphi
{ version 1.1
{                                                                           }
{ written by Jeong Byoung Sun                                               }
{            copyright  2004 - 2011                                        }
{            Email : hanuleyewa@paran.com                                   }
{                                                                           }
{ The source code is given as is. The author is not responsible             }
{ for any possible damage done due to the use of this code.                 }
{ The component can be freely used in any application. The complete         }
{ source code remains property of the author and may not be distributed,    }
{ published, given or sold in any form as such. No parts of the source      }
{ code can be included in any other component or application without        }
{ written authorization of the author.                                      }
{***************************************************************************}

unit UpdateSQLS;

interface

uses
  Classes, SysUtils, DB, SqlExpr, DBXpress;

type
  TCommitMode = (cmNone, cmRecord, cmPage);
  
  ExceptionClass = class of Exception;

  TUpdateSQLEvent = procedure (Sender: TObject; UpdateKind: TUpdateKind; var Updated: Boolean) of object;
  TBeforeGetParam = procedure (const AParamName: String; AField: TField;
      const Old: Boolean; var AValue: Variant) of object;

{ TCustomUpdateSQL }
  TCustomUpdateSQL = class(TComponent)
  private
    FSQLText: array[TUpdateKind] of TStrings;
  protected
    function GetSQLIndex(Index: Integer): TStrings; virtual;
    procedure SetSQLIndex(Index: Integer; Value: TStrings); virtual;
    procedure SQLChanged(Sender: TObject); virtual;
    function GetSQL(UpdateKind: TUpdateKind): TStrings; virtual;
    procedure SetSQL(UpdateKind: TUpdateKind; Value: TStrings); virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property SQL[UpdateKind: TUpdateKind]: TStrings read GetSQL write SetSQL;
    property ModifySQL: TStrings index 0 read GetSQLIndex write SetSQLIndex;
    property InsertSQL: TStrings index 1 read GetSQLIndex write SetSQLIndex;
    property DeleteSQL: TStrings index 2 read GetSQLIndex write SetSQLIndex;
  published

  end;

{ TSimpleUpdateSQL }
  TSimpleUpdateSQL = class(TCustomUpdateSQL)
  published
    property ModifySQL;
    property InsertSQL;
    property DeleteSQL;
  end;

{ TCustomdbxUpdateSQL }
  TCustomdbxUpdateSQL = class(TCustomUpdateSQL)
  private
    FDeltaDS: TDataSet;
    FSQLQueries: array[TUpdateKind] of TSQLQuery;
    FBeforeGetParam: TBeforeGetParam;
    FBeforeUpdate: TUpdateSQLEvent;
    FSQLConnection: TSQLConnection;
    FTransDesc: TTransactionDesc;
    function GetDataSet: TDataSet;
    procedure SetDataSet(ADataSet: TDataSet);
    function GetSQLQuery(UpdateKind: TUpdateKind): TSQLQuery;
  protected
    procedure ExecSQL(UpdateKind: TUpdateKind); virtual;
    function GetSQLIndex(Index: Integer): TStrings; override;
    procedure SetSQLIndex(Index: Integer; Value: TStrings); override;
    function GetSQL(UpdateKind: TUpdateKind): TStrings; override;
    procedure SetSQL(UpdateKind: TUpdateKind; Value: TStrings); override;
    function GetConnection: TSQLConnection; virtual;
    procedure SetConnection(AConnection: TSQLConnection); virtual;
    procedure SQLChanged(Sender: TObject); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Apply(UpdateKind: TUpdateKind);
    procedure SetParams(UpdateKind: TUpdateKind); virtual;
    function GetQuoteChar: String; virtual;
    function IsValidUpdateSQL: Boolean; virtual;
    procedure StartTransaction;
    procedure Rollback;
    procedure Commit;
    property DataSet: TDataSet read GetDataSet write SetDataSet;
    property SQLQuery[UpdateKind: TUpdateKind]: TSQLQuery read GetSQLQuery;
    property SQLConnection: TSQLConnection read GetConnection write SetConnection;
    property BeforeGetParam: TBeforeGetParam read FBeforeGetParam write FBeforeGetParam Stored True;
    property BeforeUpdate: TUpdateSQLEvent read FBeforeUpdate write FBeforeUpdate Stored True;
    property TransDesc: TTransactionDesc read FTransDesc write FTransDesc;
  published

  end;

{ TCustomdbxUpdateSQL }
  TdbxUpdateSQL = class(TCustomdbxUpdateSQL)
  private

  protected

  public

  published
    property ModifySQL;
    property InsertSQL;
    property DeleteSQL;
    property SQLConnection;
    property BeforeGetParam;
    property BeforeUpdate;
    property TransDesc;
  end;

implementation

uses
  Variants;

const
  SUpdateFailed = 'Update failed';

procedure BeginTranaction(AConection: TSQLConnection; ATranDesc: TTransactionDesc);
begin
  with AConection do
  begin
    if InTransaction then
      Rollback(ATranDesc);
    //with ATranDesc do
    //  TransactionID:= TransactionID +1;
    StartTransaction(ATranDesc);
  end;
end;

procedure RollbackTransaction(AConection: TSQLConnection; ATranDesc: TTransactionDesc);
begin
  with AConection do
  begin
    if InTransaction then
      Rollback(ATranDesc);
  end;
end;

procedure CommitTransaction(AConection: TSQLConnection; ATranDesc: TTransactionDesc);
begin
  with AConection do
  begin
    if InTransaction then
      Commit(ATranDesc);
  end;
end;

{ TCustomUpdateSQL }

constructor TCustomUpdateSQL.Create(AOwner: TComponent);
var
  UpdateKind: TUpdateKind;
begin
  inherited;
  for UpdateKind := Low(TUpdateKind) to High(TUpdateKind) do
  begin
    FSQLText[UpdateKind] := TStringList.Create;
    TStringList(FSQLText[UpdateKind]).OnChange := SQLChanged;
  end;
end;

destructor TCustomUpdateSQL.Destroy;
var
  UpdateKind: TUpdateKind;
begin
  for UpdateKind := Low(TUpdateKind) to High(TUpdateKind) do
    FSQLText[UpdateKind].Free;
  inherited;
end;

function TCustomUpdateSQL.GetSQL(UpdateKind: TUpdateKind): TStrings;
begin
  Result := FSQLText[UpdateKind];
end;

function TCustomUpdateSQL.GetSQLIndex(Index: Integer): TStrings;
begin
  Result := FSQLText[TUpdateKind(Index)];
end;

procedure TCustomUpdateSQL.SetSQL(UpdateKind: TUpdateKind; Value: TStrings);
begin
  FSQLText[UpdateKind].Assign(Value);
end;

procedure TCustomUpdateSQL.SetSQLIndex(Index: Integer; Value: TStrings);
begin
  SetSQL(TUpdateKind(Index), Value);
end;

procedure TCustomUpdateSQL.SQLChanged(Sender: TObject);
begin

end;

{ TCustomdbxUpdateSQL }

procedure TCustomdbxUpdateSQL.Apply(UpdateKind: TUpdateKind);
var
  Updated: Boolean;
begin
  Updated:= False;
  if Assigned(FBeforeUpdate) then
    FBeforeUpdate(Self, UpdateKind, Updated);
  if Updated then
    Exit;
  SetParams(UpdateKind);
  ExecSQL(UpdateKind);
end;

constructor TCustomdbxUpdateSQL.Create(AOwner: TComponent);
begin
  inherited;
  with FTransDesc do
  begin
    TransactionID:= 1;
    IsolationLevel:= xilREADCOMMITTED;
  end;
end;

destructor TCustomdbxUpdateSQL.Destroy;
begin
  inherited;
end;

procedure TCustomdbxUpdateSQL.ExecSQL(UpdateKind: TUpdateKind);
begin
  SetConnection(SQLConnection);
  with FSQLQueries[UpdateKind] do
  begin
    ExecSQL;
    if RowsAffected <> 1 then
      DatabaseError(SUpdateFailed);
  end;
end;

function TCustomdbxUpdateSQL.GetDataSet: TDataSet;
begin
  Result := FDeltaDS;
end;

function TCustomdbxUpdateSQL.GetSQL(UpdateKind: TUpdateKind): TStrings;
begin
  Result:= inherited GetSQL(UpdateKind);
end;

function TCustomdbxUpdateSQL.GetSQLIndex(Index: Integer): TStrings;
begin
  Result:= Inherited GetSQLIndex(Index);
end;

procedure TCustomdbxUpdateSQL.SetSQL(UpdateKind: TUpdateKind; Value: TStrings);
begin
  inherited;
end;

procedure TCustomdbxUpdateSQL.SetSQLIndex(Index: Integer; Value: TStrings);
begin
  inherited;
end;

procedure TCustomdbxUpdateSQL.SQLChanged(Sender: TObject);
var
  UpdateKind: TUpdateKind;
begin
  for UpdateKind := Low(TUpdateKind) to High(TUpdateKind) do
    if Sender = FSQLText[UpdateKind] then
    begin
      if Assigned(FSQLQueries[UpdateKind]) then
      begin
        FSQLQueries[UpdateKind].Params.Clear;
        FSQLQueries[UpdateKind].SQL.Assign(FSQLText[UpdateKind]);
      end;
      Break;
    end;
end;

function TCustomdbxUpdateSQL.GetSQLQuery(UpdateKind: TUpdateKind): TSQLQuery;
begin
  if not Assigned(FSQLQueries[UpdateKind]) then
  begin
    FSQLQueries[UpdateKind] := TSQLQuery.Create(Self);
    FSQLQueries[UpdateKind].SQL.Assign(FSQLText[UpdateKind]);
    FSQLQueries[UpdateKind].SQLConnection:= SQLConnection;
  end;
  Result := FSQLQueries[UpdateKind];
end;

procedure TCustomdbxUpdateSQL.SetConnection(AConnection: TSQLConnection);
var
  UpdateKind: TUpdateKind;
begin
  FSQLConnection:= AConnection;
  for UpdateKind := Low(TUpdateKind) to High(TUpdateKind) do
    if Assigned(FSQLQueries[UpdateKind]) and (FSQLQueries[UpdateKind].SQLConnection <> AConnection) then
      FSQLQueries[UpdateKind].SQLConnection:= AConnection;
end;

procedure TCustomdbxUpdateSQL.SetDataSet(ADataSet: TDataSet);
begin
  FDeltaDS := ADataSet;
end;

procedure TCustomdbxUpdateSQL.SetParams(UpdateKind: TUpdateKind);
var
  I: Integer;
  Old: Boolean;
  Param: TParam;
  PName: string;
  Field: TField;
  Value: Variant;
begin
  if not Assigned(FDeltaDS) then Exit;
  with SQLQuery[UpdateKind] do
  begin
    for I := 0 to Params.Count - 1 do
    begin
      Param := Params[I];
      if Param.ParamType = ptUnknown then
        Param.ParamType:= ptInput;
      PName := Param.Name;
      Old := SameText(Copy(PName, 1, 4), 'OLD_');
      if Old then System.Delete(PName, 1, 4);
      Field := FDeltaDS.FindField(PName);

      if Assigned(FBeforeGetParam) then
      begin
        Value:= UnAssigned;
        FBeforeGetParam(PName, Field, Old, Value);
        if Not VarIsEmpty(Value) then
        begin
          if Field = nil then
            Param.Value:= Value
          else
            Param.AssignFieldValue(Field, Value);
          Continue;
        end;
      end;

      if not Assigned(Field) then Continue;

      if Old then
        Param.AssignFieldValue(Field, Field.OldValue)
      else
      begin
        Value := Field.NewValue;
        if VarIsClear(Value) then
          Value := Field.OldValue;
        Param.AssignFieldValue(Field, Value);
      end;
    end;
  end;
end;

function TCustomdbxUpdateSQL.GetQuoteChar: String;
begin
  if Assigned(FDeltaDS) and (FDeltaDS is TCustomSQLDataSet) then
    Result:= TCustomSQLDataSet(FDeltaDS).GetQuoteChar
  else
    Result:= '';
end;

procedure TCustomdbxUpdateSQL.StartTransaction;
begin
  BeginTranaction(SQLConnection, FTransDesc);
end;

procedure TCustomdbxUpdateSQL.Commit;
begin
  CommitTransaction(SQLConnection, FTransDesc);
end;

procedure TCustomdbxUpdateSQL.Rollback;
begin
  RollbackTransaction(SQLConnection, FTransDesc);
end;

function TCustomdbxUpdateSQL.GetConnection: TSQLConnection;
begin
  Result:= FSQLConnection;
end;

function TCustomdbxUpdateSQL.IsValidUpdateSQL: Boolean;
begin
  Result:= Assigned(SQLConnection);
end;

end.
